/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * The data model containing a set of annotations for a
 * browser tab.
 * @param browser Tab associated with the annotations
 */
function AnnozillaAnnotationModel (browser)
{
    this.browser = browser;
}

AnnozillaAnnotationModel.ANNOZILLA_ANNOTATIONS_KEY = "annozilla-annotations";
AnnozillaAnnotationModel.ANNOZILLA_SERVERS_REMAINING_KEY = "annozilla-serversremaining";
AnnozillaAnnotationModel.ANNOZILLA_SERVERS_RESOLVING_COUNT_KEY = "annozilla-serversresolving";
AnnozillaAnnotationModel.ANNOZILLA_SERVERS_RESOLVING_FLAG_KEY = "annozilla-resolvingflag";

/**
 * Get the annotations for this tab.
 */
AnnozillaAnnotationModel.prototype.getAnnotations =
function ()
{
    var model = this.browser.getUserData (AnnozillaAnnotationModel.ANNOZILLA_ANNOTATIONS_KEY);
    if (model == null)
    {
        return new Object();
    }
    else
    {
        return model;
    }
};

/**
 * Set the annotations for this tab.
 */
AnnozillaAnnotationModel.prototype.setAnnotations =
function (annotations)
{
    this.browser.setUserData (
        AnnozillaAnnotationModel.ANNOZILLA_ANNOTATIONS_KEY,
        annotations,
        function (operation, key, data, source, destination) { } );
};

/**
 * Clear the annotations for this tab.
 */
AnnozillaAnnotationModel.prototype.clearAnnotations = function ()
{
    this.setAnnotations (new Object());
};

/**
 * Add an annotation to the data model.
 * "inReplyTo" and "threadRoot" should only be specified for replies
 * and can be omitted otherwise.
 */
AnnozillaAnnotationModel.prototype.addAnnotation =
function (aAnnotation, aRange, inReplyTo, threadRoot)
{

    var currentAnnotations = this.getAnnotations();
    
    currentAnnotations[aAnnotation.annotationURI] = {
        annot: aAnnotation,
        range: aRange,
        inReplyTo: inReplyTo,
        threadRoot: threadRoot        
    };

    this.setAnnotations (currentAnnotations);
};

/**
 * Update an already-stroed annotation with a DOM range.
 */
AnnozillaAnnotationModel.prototype.updateAnnotationWithRange =
function (aAnnotation, aRange)
{
    var currentAnnotations = this.getAnnotations();
    currentAnnotations[aAnnotation.annotationURI].range = aRange;
    this.setAnnotations (currentAnnotations);
};

/**
 * Delete an annotation.
 */
AnnozillaAnnotationModel.prototype.deleteAnnotation =
function (url)
{
    
    var currentAnnotations = this.getAnnotations();   
    if (currentAnnotations[url]) {
        delete currentAnnotations[url];
    }
    this.setAnnotations (currentAnnotations);

};

/**
 * Update an annotation.
 */
AnnozillaAnnotationModel.prototype.updateAnnotation =
function (updatedAnnotation)
{
    var currentAnnotations = this.getAnnotations();
    var uri = updatedAnnotation.annotationURI;
    if (currentAnnotations && currentAnnotations[uri]) {
        currentAnnotations[uri].annot = updatedAnnotation;
    }
    this.setAnnotations (currentAnnotations);

};

/**
 * Determine whether a specified annotation exists
 * in the model.
 */
AnnozillaAnnotationModel.prototype.hasAnnotation =
function (uri)
{
    var currentAnnotations = this.getAnnotations();
    return (uri != undefined &&
            currentAnnotations != undefined &&
            currentAnnotations[uri] != undefined);

};

/**
 * Get a single annotation.
 */
AnnozillaAnnotationModel.prototype.getAnnotation =
function (uri)
{
    var currentAnnotations = this.getAnnotations();
    return currentAnnotations[uri];
};

/**
 * Indicate that we are starting some operation on a
 * set of annotation servers.
 * @param serverTotal the number of servers to which this
 * operation will communicate
 * @param isLoad Optional parameter; if true, indicates that
 * the operation is a load operation
 * @return The total number of servers which we are currently
 * waiting for, across all operations
 */
AnnozillaAnnotationModel.prototype.startOperation =
function (serverTotal, isLoad)
{
    var currentCount =
        this.browser.getUserData (
           AnnozillaAnnotationModel.ANNOZILLA_SERVERS_REMAINING_KEY);
           
    if (currentCount == undefined) { currentCount = 0; }
    
    // TODO Atomicity?

    currentCount += serverTotal;

    this.browser.setUserData (
        AnnozillaAnnotationModel.ANNOZILLA_SERVERS_REMAINING_KEY,
        currentCount,
        function (operation, key, data, source, destination) { } );

    if (isLoad) {
        // Start of a load operation. Indicate that we will
        // resolve annotations
        this.browser.setUserData (
            AnnozillaAnnotationModel.ANNOZILLA_SERVERS_RESOLVING_FLAG_KEY,
            true,
            function (operation, key, data, source, destination) { } );
    }
    
    return currentCount;
};

/**
 * Indicate that an annotation sever has responded to an
 * operation.
 * @return The total number of servers which we are currently
 * waiting for, across all operations
 */
AnnozillaAnnotationModel.prototype.serverResponded =
function ()
{
    var serverCount =
        this.browser.getUserData (AnnozillaAnnotationModel.ANNOZILLA_SERVERS_REMAINING_KEY);
    serverCount--;

    this.browser.setUserData (
        AnnozillaAnnotationModel.ANNOZILLA_SERVERS_REMAINING_KEY,
        serverCount,
        function (operation, key, data, source, destination) { } );
    return serverCount;
};

/**
 * Indicate that we are starting to resolve annotations or replies
 * for an annotation server.
 * @return The total number of servers which we are currently
 * resolving annotations or replies for
 */
AnnozillaAnnotationModel.prototype.resolvingAnnotations =
function ()
{
    var currentCount =
        this.browser.getUserData (
           AnnozillaAnnotationModel.ANNOZILLA_SERVERS_RESOLVING_COUNT_KEY);
 
    if (currentCount == undefined) { currentCount = 0; }
    
    currentCount++;
    this.browser.setUserData (
        AnnozillaAnnotationModel.ANNOZILLA_SERVERS_RESOLVING_COUNT_KEY,
        currentCount,
        function (operation, key, data, source, destination) { } );
    
    return currentCount;
};

/**
 * Indicate that we have completed resolving annotations or replies
 * for an annotation server.
 * @return The total number of servers which we are currently
 * resolving annotations or replies for
 */
AnnozillaAnnotationModel.prototype.resolvingComplete =
function ()
{
    var serverCount =
        this.browser.getUserData (AnnozillaAnnotationModel.ANNOZILLA_SERVERS_RESOLVING_COUNT_KEY);
    serverCount--;
    this.browser.setUserData (
        AnnozillaAnnotationModel.ANNOZILLA_SERVERS_RESOLVING_COUNT_KEY,
        serverCount,
        function (operation, key, data, source, destination) { } );
    return serverCount;
};

/**
 * Determine whether we are still resolving annotations or replies.
 * @return true if annotations are still being resolved 
 */
AnnozillaAnnotationModel.prototype.isResolving =
function ()
{
    var serverCount =
        this.browser.getUserData
            (AnnozillaAnnotationModel.ANNOZILLA_SERVERS_RESOLVING_COUNT_KEY);
    if (serverCount == undefined) { serverCount = 0; }
    
    var resolving =
        this.browser.getUserData
            (AnnozillaAnnotationModel.ANNOZILLA_SERVERS_RESOLVING_FLAG_KEY);
    return (resolving && serverCount > 0);
};

AnnozillaAnnotationModel.prototype.stopResolving =
function ()
{
    this.browser.setUserData (
        AnnozillaAnnotationModel.ANNOZILLA_SERVERS_RESOLVING_FLAG_KEY,
        false,
        function (operation, key, data, source, destination) { } );
}